#include "iica_demo.h"
volatile uint8_t   g_iica_tx_end;             /* iica0 send data end */
volatile uint8_t   g_iica_rx_end;             /* iica0 receive data end */
uint16_t  g_iica_rx_cnt;                     /* iica1 receive data count */
uint16_t  g_iica_rx_len;                     /* iica1 receive data length */
static uint8_t tx_buf[8] = { 1, 2, 3, 4, 5, 6, 7, 8 };
static uint8_t rx_buf[8] = { 0 };
static uint16_t tx_length = 8, rx_length = 0;
I2CA_Slave_Event event;

#define _80_IICA_ADDRESS_COMPLETE                     (0x80U)
#define _00_IICA_MASTER_FLAG_CLEAR                    (0x00U)
static IICA_FRAME_t pData; /*pData used as an example,user can dinfine different paramater according to needs*/

void Iica0_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStruct = {0};
    I2CA_InitTypeDef I2CA_InitStructure;

    GPIO_PinAFConfig(GPIO_PORT6, GPIO_Pin_0, GPIO_P60, GROUP_AF_SCLAA0); //SCLA0 can be allocated to any desired pins in group0
    GPIO_PinAFConfig(GPIO_PORT6, GPIO_Pin_1, GPIO_P61, GROUP_AF_SDAA0); //SDAA0 can be allocated to any desired pins in group0

    /*SCLA0 GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_0;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;
    GPIO_InitStruct.GPIO_PuPd   = GPIO_PuPd_UP;
    GPIO_InitStruct.GPIO_OType  = GPIO_OType_OD;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_Init(GPIO_PORT6, &GPIO_InitStruct);

    /*SDAA0 GPIO CONFIG*/
    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_1;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_Level  = GPIO_Level_LOW;
    GPIO_InitStruct.GPIO_PuPd   = GPIO_PuPd_UP;
    GPIO_InitStruct.GPIO_OType  = GPIO_OType_OD;
    GPIO_InitStruct.GPIO_Ctrl   = GPIO_Control_DIG;
    GPIO_Init(GPIO_PORT6, &GPIO_InitStruct);

    I2CA_InitStructure.I2C_ClockSpeed = 800000;
    I2CA_InitStructure.I2C_Ack = I2CA_Ack_Enable;
    I2CA_InitStructure.I2C_DutyCycle = 50;
    I2CA_InitStructure.I2C_OwnAddress = 0xA0;
    I2CA_InitStructure.I2C_Mode = I2CA_Mode_SMBusSlave;
    I2CA_Init(&I2CA_InitStructure);


#ifdef I2CA_USING_INTERRUPT
    ISR_Register(IICA_IRQn, iica0_interrupt);
    INTC_EnableIRQ(IICA_IRQn);                       //enable INTIICA interrupt flag
#endif
}


/***********************************************************************************************************************
* Function Name: IICA0_SlaveSend
* @brief  This function sends data as slave mode.
* @param  adr - send address
* @param  tx_buf - transfer buffer pointer
* @param  tx_num - buffer size
* @return None
***********************************************************************************************************************/
void IICA_SlaveSend(uint8_t adr, uint8_t *const tx_buf, uint16_t tx_num)
{
    g_iica_tx_end = 0;
    pData.len = tx_num;
    pData.data = tx_buf;
    pData.flag = IICA_INT_TX;
    IICA->SVA0 = adr; /* slave address */
}
/***********************************************************************************************************************
* Function Name: IICA0_SlaveReceive
* @brief  This function receives data as slave mode.
* @param  adr - send address
* @param  tx_buf - receive buffer pointer
* @param  rx_num - buffer size
* @return None
***********************************************************************************************************************/
void IICA_SlaveReceive(uint8_t adr, uint8_t *const rx_buf, uint16_t rx_num)
{
    g_iica_rx_end = 0;
    g_iica_rx_len = 0;
    g_iica_rx_cnt = rx_num;
    pData.len = 0;
    pData.data = rx_buf;
    pData.flag = IICA_INT_RX;
    IICA->SVA0 = adr; /* slave address */
}




/***********************************************************************************************************************
* Function Name: iica0_slavehandler
* @brief   This function is IICA0 slave handler.
* @param  None
* @return None
***********************************************************************************************************************/
uint8_t addrMatchrx = 0;
uint8_t addrMatchtx = 0;
uint8_t Stop_count = 0;
__IOM uint8_t Reg = 0;
static void iica0_slavehandler()
{
    IICA_FRAME_t *pFrame = &pData;

    /* Control for stop condition */
    if (IICA->IICS0 & IICA_IICS0_SPD_Msk)
    {
        /* Get stop condition */
        Stop_count++;

        if (pFrame->flag == IICA_INT_TX && Stop_count == 2)  // send flag
        {
            IICA->IICCTL00 &= ~IICA_IICCTL00_SPIE_Msk; /* SPIE0 = 0: disable */
            g_iica_tx_end = 1; //slave_sendend
            addrMatchtx = 0;
            Stop_count = 0;
        }

        if (pFrame->flag == IICA_INT_RX)
        {
            IICA->IICCTL00 &= ~IICA_IICCTL00_SPIE_Msk; /* SPIE0 = 0: disable */
            g_iica_rx_end = 1; //slave_receiveend
            g_iica_rx_len = pFrame->len;
            addrMatchrx = 0;
            Stop_count = 0;
        }
    }
    else
    {
        if (pFrame->flag == IICA_INT_RX)
        {
            if (!addrMatchrx && IICA->IICS0 & IICA_IICS0_COI_Msk)   //ַƥ
            {
                addrMatchrx = 1;

                IICA->IICCTL00 |= IICA_IICCTL00_SPIE_Msk; /* SPIE0 = 1: enable */

                IICA->IICCTL00 |=  IICA_IICCTL00_ACKE_Msk;   /* ACKE0 = 1U: enable acknowledgment */
                /* interrupt request is generated at the eighth clock's falling edge */
                IICA->IICCTL00 &= ~IICA_IICCTL00_WTIM_Msk;

                /* cancel wait */
                IICA->IICCTL00 |= IICA_IICCTL00_WREL_Msk;
            }
            else
            {
                if (pFrame->len < g_iica_rx_cnt)
                {
                    *pFrame->data = IICA->IICA0;
                    pFrame->data++;
                    pFrame->len++;

                    IICA->IICCTL00 |= IICA_IICCTL00_WREL_Msk;   // WREL0 = 1U: cancel wait

                    if (g_iica_rx_cnt == pFrame->len)
                    {
                        IICA->IICCTL00 |= IICA_IICCTL00_WTIM_Msk;   // WTIM0 = 1:  interrupt request is generated at the ninth clock's falling edge
                        g_iica_rx_end = 1;                           // slave receive finished
                        g_iica_rx_len = pFrame->len;
                        addrMatchrx = 0;
                    }
                }
                else
                {
                    IICA->IICCTL00 |= IICA_IICCTL00_WTIM_Msk;   // WTIM0 = 1:  interrupt request is generated at the ninth clock's falling edge
                    IICA->IICCTL00 |=  IICA_IICCTL00_WREL_Msk;   // WREL0 = 1U: cancel wait
                }
            }
        }

        if (pFrame->flag == IICA_INT_TX)
        {
            if ((addrMatchtx < 2) && (IICA->IICS0 & IICA_IICS0_COI_Msk)) //ַƥ
            {
                addrMatchtx ++;

                if (addrMatchtx == 1)
                {
                    IICA->SVA0 |= 0x01;    //һȶ
                }
                else if (addrMatchtx == 2)
                {
                    Reg = IICA->IICA0;
                    IICA->SVA0 &= ~0x01;
                }

                if (!(IICA->IICCTL00 & IICA_IICCTL00_SPIE_Msk))
                {
                    /* SPIE0 = 1: enable I2CA */
                    IICA->IICCTL00 |= IICA_IICCTL00_SPIE_Msk;
                }

                /* Sending or receiving */
                if (IICA->IICS0 & IICA_IICS0_TRC_Msk)    //send status
                {
                    IICA->IICCTL00 |= IICA_IICCTL00_WTIM_Msk; //9ʱӣʱΪ͵ƽ

                }
                else
                {
                    IICA->IICCTL00 &= ~IICA_IICCTL00_WTIM_Msk;    // interrupt request is generated at the eighth clock's falling edge
                    IICA->IICCTL00 |= IICA_IICCTL00_WREL_Msk;     // cancel wait
                }
            }
            else
            {
                if (IICA->IICS0 & IICA_IICS0_TRC_Msk)  /* TRC0 == 1 */
                {
                    if (pFrame->len > 0U)
                    {
                        IICA->IICA0 = *pFrame->data;
                        pFrame->data++;
                        pFrame->len--;
                    }
                    else
                    {
                        IICA->IICA0 = 0xFF;
                        g_iica_tx_end = 1;
                    }
                }
            }

        }

    }
}

void iica0_interrupt(void *msg)
{
    INTC_ClearPendingIRQ(IICA_IRQn);

//	iica0_slavehandler((void *)msg);
	event = I2CA_Slave_TransmitReceive_IT(tx_buf, &tx_length, rx_buf, &rx_length, 8);

}
