/***********************************************************************************************************************
* @file    lin.c
* @brief   This file implements device driver for LIN Send and Receive.
* @version 1.0.0
* @date    2019/12/24
***********************************************************************************************************************/
#include "lin_protocol.h"
#include "uart_demo.h"
#include "intp_demo.h"
volatile uint8_t *gp_uart0_rx_address;       /* uart0 receive buffer address */
volatile uint16_t g_uart0_rx_count;          /* uart0 receive data number */
volatile uint16_t g_uart0_rx_length;         /* uart0 receive data length */
volatile uint8_t g_uart_tx_end, g_uart_rx_end;
volatile uint32_t g_inttm03Taken = 0UL;
volatile uint32_t g_tm40_ch3_width = 0UL;
const uint16_t Sps_table[16] = {1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768};

#define LIN_BAUDRATE     (19200)        /* baudrate: 19200bps */

lin_frame_t txFrame, rxFrame;
lin_state_t lin_state;
uint32_t break_timeout, rx_timeout;
uint8_t lin_check_timeout;
extern uint8_t lin_tx_buf[9];
UART_FRAME_t pLindata;
//жϴ
uint8_t Lin_CheckPID(uint8_t id)
{
	uint8_t pid ;
	uint8_t P0 ;
	uint8_t P1 ;
	
	P0 = (((id)^(id>>1)^(id>>2)^(id>>4))&0x01)<<6 ;
	P1 = ((~((id>>1)^(id>>3)^(id>>4)^(id>>5)))&0x01)<<7 ;
	
	pid = id|P0|P1 ;
	
	return pid ;
} 

uint8_t Lin_Checksum(uint8_t id , uint8_t data[])
{
	uint8_t t ;
	uint16_t sum ;

	sum = data[0];
	if(id == 0x3c)			// ֡þУ
	{
		
		for(t=1;t<8;t++)
		{
			sum += data[t];
			if(sum&0xff00)
			{
				sum&=0x00ff;
				sum+=1;
			}
		}
		sum = ~sum;	
		
		return (uint8_t)sum ;
	}
	
	for(t=1;t<8;t++)
	{
		sum += data[t];
		if(sum&0xff00)
		{
			sum&=0x00ff;
			sum+=1;
		}
	}
	sum+=Lin_CheckPID(id);
	if(sum&0xff00)
	{
		sum&=0x00ff;
		sum+=1;
	}
	sum = ~sum;	
	return (uint8_t)sum ;
}


void LIN_SleepIO_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStruct = {0};

    GPIO_InitStruct.GPIO_Pin    = GPIO_Pin_0;
    GPIO_InitStruct.GPIO_Mode   = GPIO_Mode_OUT;
    GPIO_InitStruct.GPIO_OType  = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_Level   = GPIO_Level_LOW;	
    GPIO_InitStruct.GPIO_Ctrl  = GPIO_Control_DIG;
    GPIO_Init(GPIO_PORT7, &GPIO_InitStruct);
}
/***********************************************************************************************************************
* Function Name: LIN_SetConfig
* @brief  This function sets the configure of LIN.
* @param  None
* @return None
***********************************************************************************************************************/
void LIN_SetConfig(void)
{
    MISC->ISC = 0x03; /* RXD0 is used as TI03 and INTP0 input */

	CGC_PER0PeriphClockCmd(CGC_PER0Periph_TIM40,ENABLE);/* enables input clock supply */
    TM40->TPS = 0x0000;

    /* INTP0 setting */
	Intp_Init();
    /* UART0 setting */
    Uart0_Init(LIN_BAUDRATE);
    ISR_Register(ST0_IRQn, uart0_lin_send);     //linжϷ·ע
    ISR_Register(SR0_IRQn, uart0_lin_receive);  //linжϷ·ע

    ISR_Register(TM03_IRQn, tim40_channel3_interrupt);//ʱжϷע
	INTC_DisableIRQ(TM03_IRQn);
}

/***********************************************************************************************************************
* Function Name: LIN_ReceiveBreak
* @brief  This function implements measure the break field of LIN.
* @param  None
* @return None
***********************************************************************************************************************/
void LIN_ReceiveBreak(void)
{
	TIM_Stop(TM40,TIM_Index_Channel3);
    INTC_ClearPendingIRQ(TM03_IRQn);  /* clear interrupt flag */

    /* Channel 3 is used to measure input pulse width */
//    TM40->TMR03 = _0000_TM4_CLOCK_SELECT_CKM0 | _0000_TM4_CLOCK_MODE_CKS | _0200_TM4_TRIGGER_TIMN_BOTH |
//                  _0080_TM4_TIMN_EDGE_BOTH_LOW | _000C_TM4_MODE_HIGHLOW_MEASURE;
    TM40->TMR[3] = 0x0000 | 0x0000 | 0x0200 | 0x0080 | 0x000C;//öʱͨ3Ϊ͵ƽģʽ

    INTC_EnableIRQ(TM03_IRQn);  /* enable interrupt */
	TIM_Start(TM40,TIM_Index_Channel3);
	
    g_inttm03Taken = 0;
}

/***********************************************************************************************************************
* Function Name: LIN_GetBaudRate
* @brief  This function gets the baud rate by measure the sync field of LIN.
* @param  None
* @return the setting value of baudrate
***********************************************************************************************************************/
uint16_t LIN_GetBaudRate(void)
{
    uint32_t period = 0;
    uint8_t  tmp;

    while(g_inttm03Taken < 2);
    period = g_tm40_ch3_width;
    /* calculate baudrate value */
    tmp = period / Sps_table[UART_PRS] / 4 - 1;

    return (tmp << 9);
}
/***********************************************************************************************************************
* Function Name: LIN_ReceiveSync
* @brief  This function implements measure the sync field of LIN.
* @param  None
* @return None
***********************************************************************************************************************/
void LIN_ReceiveSync(void)
{
	TIM_Stop(TM40,TIM_Index_Channel3);
    INTC_ClearPendingIRQ(TM03_IRQn);  /* clear interrupt flag */

    /* Channel 3 is used to measure input pulse interval at falling edge */
//    TM40->TMR03 = _0000_TM4_CLOCK_SELECT_CKM0 | _0000_TM4_CLOCK_MODE_CKS | _0100_TM4_TRIGGER_TIMN_VALID |
//                  _0000_TM4_TIMN_EDGE_FALLING | _0004_TM4_MODE_CAPTURE | _0000_TM4_START_INT_UNUSED;
    TM40->TMR[3] = 0x0000 | 0x0000 | 0x0100 | 0x0000 | 0x0004;//öʱͨ3Ϊ
	
    INTC_EnableIRQ(TM03_IRQn);  /* enable interrupt */
	TIM_Start(TM40,TIM_Index_Channel3);

    g_inttm03Taken = 0;
}




void lin_callback_error(uint8_t err_type)
{
    //user edit here when appear error
}

void LIN_Receive(uint8_t * rx_buf, uint8_t rx_num)
{
	INTC_ClearPendingIRQ(SR0_IRQn);  /* enable interrupt */		//laidi ˱־ʹܺһж
    INTC_EnableIRQ(SR0_IRQn);  /* enable interrupt */
    g_uart0_rx_count = 0U;
    g_uart0_rx_length = rx_num;
    gp_uart0_rx_address = rx_buf;
}

void LIN_Send(uint8_t * tx_buf, uint8_t tx_num)
{
    pLindata.data = tx_buf;
    pLindata.len = tx_num;

    INTC_SetPendingIRQ(ST0_IRQn);
}

/*****************************************************************************
* Function Name: uart0_lin_receive
* @brief  UART0 Receive interrupt service routine
* @param  None
* @return None
*****************************************************************************/
void uart0_lin_send()
{
    UART_FRAME_t *pFrame = &pLindata;

    INTC_ClearPendingIRQ(ST0_IRQn);

    if ((pFrame->len > 0U) && pFrame->data)
    {
        UART0_TX = *pFrame->data;
        pFrame->data++;
        pFrame->len --;
    }
    else  //send finished
    {
		g_uart_tx_end = 1;
    }
}


/*****************************************************************************
* Function Name: uart0_lin_receive
* @brief  UART0 Receive interrupt service routine
* @param  None
* @return None
*****************************************************************************/
void uart0_lin_receive()
{
    volatile uint8_t rx_data;
    volatile uint8_t err_type;
	volatile uint8_t ID;
	
    INTC_ClearPendingIRQ(SR0_IRQn);
    err_type = UART_GetErrStaus(UART0, UART_FLAG_FEF | UART_FLAG_PEF | UART_FLAG_OVF);
	SCI_ClearFlag(UART0, err_type);

    if (err_type)
    {
        lin_callback_error(err_type);
    }

    rx_data = UART0_RX;

    if (g_uart0_rx_length > g_uart0_rx_count)
    {
        *gp_uart0_rx_address = rx_data;
        gp_uart0_rx_address++;
        g_uart0_rx_count++;
			
		//ж
		if(g_uart0_rx_count == 1 && rx_data != 0x55)	//sync
		{
		}
		if(g_uart0_rx_count == 2)	//PID
		{
			ID = rx_data & 0x3f;
			if(rx_data != Lin_CheckPID(ID))	//check PID error
			{
				return;
			}
			if(ID == 0x06)	//ҪظID
			{
				//
				INTC_DisableIRQ(SR0_IRQn);  /* disable interrupt */
				g_uart_tx_end = 0;
				LIN_Send(lin_tx_buf, 9);
				lin_state = LIN_TX;
			}
		}

        if (g_uart0_rx_length == g_uart0_rx_count)
        {
			g_uart_rx_end = 1;
        }
	}
}


/***********************************************************************************************************************
* Function Name: tim40_channel3_interrupt
* @brief  TIM40 Channel interrupt service routine
* @param  msg
* @return None
***********************************************************************************************************************/
void tim40_channel3_interrupt()
{
    INTC_ClearPendingIRQ(TM03_IRQn);    // clear INTTM00 interrupt flag
    g_inttm03Taken++;
    if (1U == (TM40->TSR[3] & 0x0001))    /* overflow occurs */
    {
        g_tm40_ch3_width = (uint32_t)(TM40->TDR1[1] + 1UL) + 0x10000UL;
    }
    else
    {
        g_tm40_ch3_width = (uint32_t)(TM40->TDR1[1] + 1UL);
    }
		
	//жжϼ
	if(g_tm40_ch3_width > 11 * 2500)		//(SystemCoreClock / LIN_BAUDRATE)=2500
	{
		LIN_Receive((uint8_t *)&rxFrame, sizeof(rxFrame));
		lin_state = LIN_RX;
		rx_timeout = 0;
		
		//رնʱʼ㳬ʱ
		TIM_Stop(TM40,TIM_Index_Channel3);//ضʱ,ڴ״̬
		lin_check_timeout = 1;
	}else
	{
		lin_state = IDLE;
	}

}
